/*
 * $Id: parse.c,v 1.4 2000/05/10 18:07:38 rgb Exp $
 *
 * procstatd - Copyright (c) 1999 by Robert G. Brown, rgb@phy.duke.edu
 *         GPL version 2b (b for beverage) granted.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * procstatd - A daemon to extract statistics from /proc/stat and publish them
 *         on demand via a socket connection or broadcast.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#define BUFLEN 1024

/*
 * parse() is a reusable routine to break up a string into char[32]
 * fields.  Anything past 32 characters is truncated.
 */

int parse(inbuffer,outfields,maxfields,maxfieldlength)
char *inbuffer;
char **outfields;
int maxfieldlength;
int maxfields;
{

 char delim[7],*nextval;
 int i = 0;
/* 
 * Permit blank, tab, or comma separators anywhere we need to parse
 * a line.
 */
 delim[0] = ' ';                /* blanks */
 delim[1] = (char) 9;           /* tab */
 delim[2] = ',';                /* comma */
 delim[3] = (char) 10;		/* LF */
 delim[4] = (char) 13;		/* CR */
 delim[5] = ':';		/* : needed to parse /proc/net/dev or passwd */
 delim[6] = (char) NULL;        /* terminator */

 nextval = strtok(inbuffer,delim);
 strncpy(outfields[i++],nextval,maxfieldlength);

 while(i < maxfields){
   nextval = strtok((char *) NULL, delim);
   if(nextval == (char *)NULL) break;
   strncpy(outfields[i++], nextval,maxfieldlength);
 }

 return i;

}

/*
 * parse_chips() is a special purpose routine required to parse the
 * return from /proc/sys/dev/sensors/chips (only).  The second field
 * MIGHT be isa or smbus and a unique identifier of a "good" sensor.
 */

int parse_chips(inbuffer,outfields,maxfields,maxfieldlength)
char *inbuffer;
char **outfields;
int maxfieldlength;
int maxfields;
{

 char delim[2],*nextval;
 int i = 0;
/* 
 * The delimiter in question is a "-", to parse e.g. "w83782d-isa-0290".
 */
 delim[0] = '-';                /* blanks */
 delim[1] = (char) NULL;        /* terminator */

 nextval = strtok(inbuffer,delim);
 strncpy(outfields[i++],nextval,maxfieldlength);

 while(i < maxfields){
   nextval = strtok((char *) NULL, delim);
   if(nextval == (char *)NULL) break;
   strncpy(outfields[i++], nextval,maxfieldlength);
 }

 return i;

}

