/*
 * $Id: proc_sensors.c,v 1.7 2000/05/10 18:07:38 rgb Exp $
 *
 * procstatd - Copyright (c) 1999 by Robert G. Brown, rgb@phy.duke.edu
 *         GPL version 2b (b for beverage) granted.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * procstatd - A daemon to extract statistics from /proc/stat and publish them
 *         on demand via a socket connection or broadcast.
 */

#include "procstatd.h"

/*
 * This is the sensors parsing routine for lm-sensors 2.5.0 (hopefully
 * it will work for later versions, but I have no control over that).
 * It reads the available chip interfaces from
 *    /proc/sys/dev/sensors/chips
 * and looks for as much as it can find of the following constellation
 * of readings:  fan1, fan2, fan3 and temp1, temp2, temp3.  At this
 * moment I don't see how to do voltages portably without linking in
 * libsensors.a, which sort of voids the point of a /proc interface.
 */

char chippath[BUFLEN];	/* Needed only in proc_sensors.c */

void init_proc_sensors()
{

 int i,n,numfields,chipflag = 0;
 FILE *chips;
 char **chipfields;

 /*
  * Allocate a major space for parsing the name of the chip separated
  * by the "-" delimiter.  Grrr.  Perl makes all this so easy with
  * regular expressions and the split command.
  */
 chipfields = (char **) malloc((size_t) (MAXFIELDNUMBER*sizeof(char*)));
 for(n = 0;n < MAXFIELDNUMBER;n++) {
   chipfields[n] = (char *) malloc((size_t)(BUFLEN*sizeof(char)));
 }

 /* 
  * To find the fields we wish to monitor, we first have to see if
  * there is a sensors device at all and determine the chip interface.
  */
 chips = fopen("/proc/sys/dev/sensors/chips","r");

 /*
  * If the open succeeded, the file descriptor is nonzero.  We then
  * must read in the lines and look for the name of the/a viable interface.
  * We will assume that this is the the isa interface (presumably the 
  * fastest) but will look for an smbus interface too.  Unfortunately
  * I have no way of testing my code here generally, so I'll just have
  * to wait for complaints if it fails.  We start by setting the chippath
  * to the base directory.
  */

 if(chips){
   while(chipflag == 0){

     /* Normal EOF causes break from while loop */
     if((fgets(statbuf,BUFLEN,chips) == NULL)) break;

     /* parse the line into fields and tell us what the chipname is. */
     numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

     /*
      * This is a mess, sorry.  fields[1] is now (I believe) a string
      * like "w83782d-isa-0-2d" or "eeprom-i2c-0-52".  If it has an
      * "isa" in there it is likely a "good" sensor.  I'd guess ditto if
      * if it has an "smbus".  So we'll parse_chips() to break field[1]
      * up into fields and look for it.  Sigh.
      */
     strncpy(chippath,"/proc/sys/dev/sensors/",sizeof(chippath));
     strncat(chippath,fields[1],sizeof(chippath));
     strncat(chippath,"/",sizeof(chippath));
     if(verbose){
       printf("# proc_sensors: Testing chippath %s\n",chippath);
     }
     numfields = parse_chips(fields[1],chipfields,MAXFIELDNUMBER,BUFLEN);

     for(i=0;i<numfields;i++){
       if(verbose){
         printf("# proc_sensors: Testing field %s\n",chipfields[i]);
       }
       if((strlen(chipfields[i]) == 3) && strncmp(chipfields[i],"isa",3) == 0){
         if(verbose){
           printf("# proc_sensors: sensors path %s discovered\n",chipfields[i]);
         }
         chipflag = 1;
         break;
       } else if((strlen(chipfields[i]) == 5) && strncmp(chipfields[i],"smbus",3) == 0){
         if(verbose){
           printf("# proc_sensors: sensors path %s discovered\n",chipfields[i]);
         }
         chipflag = 1;
         break;
       }
     }
   } /* End while(true) */
 } else {
   return;
 }

 if(chipflag == 0) return;

 /*
  * At this point chippath should be a valid path, we hope.
  */
 if(verbose){
   printf("# proc_sensors: Pathfinding finished, continuing with %s\n",chippath);
 }

 /*
  * This now deviates from previous code.  There is no real
  * file descriptor for sensors data because they now put each
  * field in its own file (Damn!  So inefficient!  So much overhead!)
  * I therefore use (e.g.) stat_fd[PROC_SENSORS_TEMP1],
  * stat_fd[PROC_SENSORS_TEMP2] .... [PROC_SENSORS_FAN3]
  * each separately and record the ones that exist. Needless to say, 
  * this is not happy making, especially when their names and numbers
  * vary according to the whim of the lm_sensors folks.
  *
  * When an open succeeds, the file descriptor is nonzero.  We then
  * initialize all stats derived from /proc/stat and put values into
  * stats[FIELD].{name,source,avail,current,previous,rate} as usual.
  */

 /* TEMP */
 strncpy(sensor,chippath,sizeof(sensor));
 strncat(sensor,"temp",sizeof(sensor));
 stat_fd[PROC_SENSORS_TEMP] = fopen(sensor,"r");
 if(stat_fd[PROC_SENSORS_TEMP]){

   while(TRUE){
     /* Normal EOF causes break from while loop */
     if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_TEMP]) == NULL)) break;
     /* parse the line into fields */
     numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

     /* TEMP */
     sprintf(stats[TEMP].name,"temp");		/* Label it. */
     stats[TEMP].source = PROC_SENSORS_TEMP1;	/* Tag its source for xref */
     stats[TEMP].avail = 1;			/* Yes, we found it */
     stats[TEMP].current = atof(fields[2]);	/* current value */
   } /* End while(true) */
 }

 /* TEMP1 */
 strncpy(sensor,chippath,sizeof(sensor));
 strncat(sensor,"temp1",sizeof(sensor));
 stat_fd[PROC_SENSORS_TEMP1] = fopen(sensor,"r");
 if(stat_fd[PROC_SENSORS_TEMP1]){

   while(TRUE){
     /* Normal EOF causes break from while loop */
     if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_TEMP1]) == NULL)) break;
     /* parse the line into fields */
     numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

     /* TEMP1 */
     sprintf(stats[TEMP1].name,"temp1");		/* Label it. */
     stats[TEMP1].source = PROC_SENSORS_TEMP1;		/* Tag its source for xref */
     stats[TEMP1].avail = 1;			/* Yes, we found it */
     stats[TEMP1].current = atof(fields[2]);	/* current value */
   } /* End while(true) */
 }

 /* TEMP2 */
 strncpy(sensor,chippath,sizeof(sensor));
 strncat(sensor,"temp2",sizeof(sensor));
 stat_fd[PROC_SENSORS_TEMP2] = fopen(sensor,"r");
 if(stat_fd[PROC_SENSORS_TEMP2]){

   while(TRUE){
     /* Normal EOF causes break from while loop */
     if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_TEMP2]) == NULL)) break;
     /* parse the line into fields */
     numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

     /* TEMP2 */
     sprintf(stats[TEMP2].name,"temp2");	/* Label it. */
     stats[TEMP2].source = PROC_SENSORS_TEMP2;		/* Tag its source for xref */
     stats[TEMP2].avail = 1;			/* Yes, we found it */
     stats[TEMP2].current = atof(fields[2]);	/* current value */
   } /* End while(true) */
 }

 /* TEMP3 */
 strncpy(sensor,chippath,sizeof(sensor));
 strncat(sensor,"temp3",sizeof(sensor));
 stat_fd[PROC_SENSORS_TEMP3] = fopen(sensor,"r");
 if(stat_fd[PROC_SENSORS_TEMP3]){

   while(TRUE){
     /* Normal EOF causes break from while loop */
     if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_TEMP3]) == NULL)) break;
     /* parse the line into fields */
     numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

     /* TEMP3 */
     sprintf(stats[TEMP3].name,"temp3");	/* Label it. */
     stats[TEMP3].source = PROC_SENSORS_TEMP3;		/* Tag its source for xref */
     stats[TEMP3].avail = 1;			/* Yes, we found it */
     stats[TEMP3].current = atof(fields[2]);	/* current value */
   } /* End while(true) */
 }

 /* FAN */
 strncpy(sensor,chippath,sizeof(sensor));
 strncat(sensor,"fan",sizeof(sensor));
 stat_fd[PROC_SENSORS_FAN] = fopen(sensor,"r");
 if(stat_fd[PROC_SENSORS_FAN]){

   while(TRUE){
     /* Normal EOF causes break from while loop */
     if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_FAN]) == NULL)) break;
     /* parse the line into fields */
     numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

     /* FAN */
     sprintf(stats[FAN].name,"fan");		/* Label it. */
     stats[FAN].source = PROC_SENSORS_FAN;	/* Tag its source for xref */
     stats[FAN].avail = 1;			/* Yes, we found it */
     stats[FAN].current = atof(fields[1]);	/* current value */
   } /* End while(true) */
 }

 /* FAN1 */
 strncpy(sensor,chippath,sizeof(sensor));
 strncat(sensor,"fan1",sizeof(sensor));
 stat_fd[PROC_SENSORS_FAN1] = fopen(sensor,"r");
 if(stat_fd[PROC_SENSORS_FAN1]){

   while(TRUE){
     /* Normal EOF causes break from while loop */
     if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_FAN1]) == NULL)) break;
     /* parse the line into fields */
     numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

     /* FAN1 */
     sprintf(stats[FAN1].name,"fan1");		/* Label it. */
     stats[FAN1].source = PROC_SENSORS_FAN1;	/* Tag its source for xref */
     stats[FAN1].avail = 1;			/* Yes, we found it */
     stats[FAN1].current = atof(fields[1]);	/* current value */
   } /* End while(true) */
 }

 /* FAN2 */
 strncpy(sensor,chippath,sizeof(sensor));
 strncat(sensor,"fan2",sizeof(sensor));
 stat_fd[PROC_SENSORS_FAN2] = fopen(sensor,"r");
 if(stat_fd[PROC_SENSORS_FAN2]){

   while(TRUE){
     /* Normal EOF causes break from while loop */
     if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_FAN2]) == NULL)) break;
     /* parse the line into fields */
     numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

     /* FAN2 */
     sprintf(stats[FAN2].name,"fan2");		/* Label it. */
     stats[FAN2].source = PROC_SENSORS_FAN2;	/* Tag its source for xref */
     stats[FAN2].avail = 1;			/* Yes, we found it */
     stats[FAN2].current = atof(fields[1]);	/* current value */
   } /* End while(true) */
 }

 /* FAN3 */
 strncpy(sensor,chippath,sizeof(sensor));
 strncat(sensor,"fan3",sizeof(sensor));
 stat_fd[PROC_SENSORS_FAN3] = fopen(sensor,"r");
 if(stat_fd[PROC_SENSORS_FAN3]){

   while(TRUE){
     /* Normal EOF causes break from while loop */
     if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_FAN3]) == NULL)) break;
     /* parse the line into fields */
     numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

     /* FAN3 */
     sprintf(stats[FAN3].name,"fan3");		/* Label it. */
     stats[FAN3].source = PROC_SENSORS_FAN3;	/* Tag its source for xref */
     stats[FAN3].avail = 1;			/* Yes, we found it */
     stats[FAN3].current = atof(fields[1]);	/* current value */
   } /* End while(true) */
 }

} /* End init_proc_sensors() */


/*
 *==================================================================
 * get_proc_sensors gets values from the sensors after their original
 * initialization.
 *==================================================================
 */
void get_proc_sensors()
{

 int i,numfields;


 /* 
  * Now, for a clever trick.  We'll reset the files without actually
  * closing or reopening them.  Perhaps we can save the overhead of
  * an open/close (presumed relatively large, as one has to stat the
  * files in question on EACH open).
  */

 /* TEMP */
 errno = 0;
 if(stat_fd[PROC_SENSORS_TEMP]){
    rewind(stat_fd[PROC_SENSORS_TEMP]);	/* void, so tough to check errors */
 } else {
    return;
 }
 if(errno == EBADF){
   if(daemonmode == FORK) fprintf(stderr,"Error: The /proc/sensors file descriptor/stream is not seekable.\n");
   if(daemonmode == FORK) fprintf(stderr,"Closing and reopening /proc/sensors.\n");
   fclose(stat_fd[PROC_SENSORS_TEMP]); 
   strncpy(sensor,chippath,sizeof(sensor));
   strncat(sensor,"temp",sizeof(sensor));
   stat_fd[PROC_SENSORS_TEMP] = fopen(sensor,"r");
 }

 while(TRUE){
   /* Normal EOF causes break from while loop */
   if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_TEMP]) == NULL)) break;
   /* parse the line into fields */
   numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

   /* TEMP */
   stats[TEMP].previous = stats[TEMP].current;	/* previous value */
   stats[TEMP].current = atof(fields[2]);	/* current value */
 } /* End while(true) */

 /* TEMP1 */
 errno = 0;
 if(stat_fd[PROC_SENSORS_TEMP1]){
    rewind(stat_fd[PROC_SENSORS_TEMP1]);	/* void, so tough to check errors */
 } else {
    return;
 }
 if(errno == EBADF){
   if(daemonmode == FORK) fprintf(stderr,"Error: The /proc/sensors file descriptor/stream is not seekable.\n");
   if(daemonmode == FORK) fprintf(stderr,"Closing and reopening /proc/sensors.\n");
   fclose(stat_fd[PROC_SENSORS_TEMP1]); 
   strncpy(sensor,chippath,sizeof(sensor));
   strncat(sensor,"temp1",sizeof(sensor));
   stat_fd[PROC_SENSORS_TEMP1] = fopen(sensor,"r");
 }

 while(TRUE){
   /* Normal EOF causes break from while loop */
   if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_TEMP1]) == NULL)) break;
   /* parse the line into fields */
   numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

   /* TEMP1 */
   stats[TEMP1].previous = stats[TEMP1].current;	/* previous value */
   stats[TEMP1].current = atof(fields[2]);	/* current value */
 } /* End while(true) */



 /* TEMP2 */
 errno = 0;
 if(stat_fd[PROC_SENSORS_TEMP2]){
    rewind(stat_fd[PROC_SENSORS_TEMP2]);	/* void, so tough to check errors */
 } else {
    return;
 }
 if(errno == EBADF){
   if(daemonmode == FORK) fprintf(stderr,"Error: The /proc/sensors file descriptor/stream is not seekable.\n");
   if(daemonmode == FORK) fprintf(stderr,"Closing and reopening /proc/sensors.\n");
   fclose(stat_fd[PROC_SENSORS_TEMP2]); 
   strncpy(sensor,chippath,sizeof(sensor));
   strncat(sensor,"temp2",sizeof(sensor));
   stat_fd[PROC_SENSORS_TEMP2] = fopen(sensor,"r");
 }

 while(TRUE){
   /* Normal EOF causes break from while loop */
   if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_TEMP2]) == NULL)) break;
   /* parse the line into fields */
   numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

   /* TEMP2 */
   stats[TEMP2].previous = stats[TEMP2].current;	/* previous value */
   stats[TEMP2].current = atof(fields[2]);	/* current value */
 } /* End while(true) */



 /* TEMP3 */
 errno = 0;
 if(stat_fd[PROC_SENSORS_TEMP3]){
    rewind(stat_fd[PROC_SENSORS_TEMP3]);	/* void, so tough to check errors */
 } else {
    return;
 }
 if(errno == EBADF){
   if(daemonmode == FORK) fprintf(stderr,"Error: The /proc/sensors file descriptor/stream is not seekable.\n");
   if(daemonmode == FORK) fprintf(stderr,"Closing and reopening /proc/sensors.\n");
   fclose(stat_fd[PROC_SENSORS_TEMP3]); 
   strncpy(sensor,chippath,sizeof(sensor));
   strncat(sensor,"temp3",sizeof(sensor));
   stat_fd[PROC_SENSORS_TEMP3] = fopen(sensor,"r");
 }

 while(TRUE){
   /* Normal EOF causes break from while loop */
   if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_TEMP3]) == NULL)) break;
   /* parse the line into fields */
   numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

   /* TEMP3 */
   stats[TEMP3].previous = stats[TEMP3].current;	/* previous value */
   stats[TEMP3].current = atof(fields[2]);	/* current value */
 } /* End while(true) */



 /* FAN */
 errno = 0;
 if(stat_fd[PROC_SENSORS_FAN]){
    rewind(stat_fd[PROC_SENSORS_FAN]);	/* void, so tough to check errors */
 } else {
    return;
 }
 if(errno == EBADF){
   if(daemonmode == FORK) fprintf(stderr,"Error: The /proc/sensors file descriptor/stream is not seekable.\n");
   if(daemonmode == FORK) fprintf(stderr,"Closing and reopening /proc/sensors.\n");
   fclose(stat_fd[PROC_SENSORS_FAN]); 
   strncpy(sensor,chippath,sizeof(sensor));
   strncat(sensor,"fan",sizeof(sensor));
   stat_fd[PROC_SENSORS_FAN] = fopen(sensor,"r");
 }

 while(TRUE){
   /* Normal EOF causes break from while loop */
   if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_FAN]) == NULL)) break;
   /* parse the line into fields */
   numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

   /* FAN */
   stats[FAN].previous = stats[FAN].current;	/* previous value */
   stats[FAN].current = atof(fields[1]);	/* current value */
 } /* End while(true) */



 /* FAN1 */
 errno = 0;
 if(stat_fd[PROC_SENSORS_FAN1]){
    rewind(stat_fd[PROC_SENSORS_FAN1]);	/* void, so tough to check errors */
 } else {
    return;
 }
 if(errno == EBADF){
   if(daemonmode == FORK) fprintf(stderr,"Error: The /proc/sensors file descriptor/stream is not seekable.\n");
   if(daemonmode == FORK) fprintf(stderr,"Closing and reopening /proc/sensors.\n");
   fclose(stat_fd[PROC_SENSORS_FAN1]); 
   strncpy(sensor,chippath,sizeof(sensor));
   strncat(sensor,"fan1",sizeof(sensor));
   stat_fd[PROC_SENSORS_FAN1] = fopen(sensor,"r");
 }

 while(TRUE){
   /* Normal EOF causes break from while loop */
   if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_FAN1]) == NULL)) break;
   /* parse the line into fields */
   numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

   /* FAN1 */
   stats[FAN1].previous = stats[FAN1].current;	/* previous value */
   stats[FAN1].current = atof(fields[1]);	/* current value */
 } /* End while(true) */



 /* FAN2 */
 errno = 0;
 if(stat_fd[PROC_SENSORS_FAN2]){
    rewind(stat_fd[PROC_SENSORS_FAN2]);	/* void, so tough to check errors */
 } else {
    return;
 }
 if(errno == EBADF){
   if(daemonmode == FORK) fprintf(stderr,"Error: The /proc/sensors file descriptor/stream is not seekable.\n");
   if(daemonmode == FORK) fprintf(stderr,"Closing and reopening /proc/sensors.\n");
   fclose(stat_fd[PROC_SENSORS_FAN2]); 
   strncpy(sensor,chippath,sizeof(sensor));
   strncat(sensor,"fan2",sizeof(sensor));
   stat_fd[PROC_SENSORS_FAN2] = fopen(sensor,"r");
 }

 while(TRUE){
   /* Normal EOF causes break from while loop */
   if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_FAN2]) == NULL)) break;
   /* parse the line into fields */
   numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

   /* FAN2 */
   stats[FAN2].previous = stats[FAN2].current;	/* previous value */
   stats[FAN2].current = atof(fields[1]);	/* current value */
 } /* End while(true) */



 /* FAN3 */
 errno = 0;
 if(stat_fd[PROC_SENSORS_FAN3]){
    rewind(stat_fd[PROC_SENSORS_FAN3]);	/* void, so tough to check errors */
 } else {
    return;
 }
 if(errno == EBADF){
   if(daemonmode == FORK) fprintf(stderr,"Error: The /proc/sensors file descriptor/stream is not seekable.\n");
   if(daemonmode == FORK) fprintf(stderr,"Closing and reopening /proc/sensors.\n");
   fclose(stat_fd[PROC_SENSORS_FAN3]); 
   strncpy(sensor,chippath,sizeof(sensor));
   strncat(sensor,"fan3",sizeof(sensor));
   stat_fd[PROC_SENSORS_FAN3] = fopen(sensor,"r");
 }

 while(TRUE){
   /* Normal EOF causes break from while loop */
   if((fgets(statbuf,BUFLEN,stat_fd[PROC_SENSORS_FAN3]) == NULL)) break;
   /* parse the line into fields */
   numfields = parse(statbuf,fields,MAXFIELDNUMBER,BUFLEN);

   /* FAN3 */
   stats[FAN3].previous = stats[FAN3].current;	/* previous value */
   stats[FAN3].current = atof(fields[1]);	/* current value */
 } /* End while(true) */

} /* End get_proc_sensors() */



/*
 *==================================================================
 * eval_proc_sensors evaluates the values by value type (rate or
 * simple value) after a get_proc_sensors() call.
 *==================================================================
 */
void eval_proc_sensors()
{

 int i,numfields;


 for(i=0;i<N_STATS;i++){

   switch(i) {
     /* 
      * TEMP is a straight value.
      */
     case TEMP:
       stats[i].rate = stats[i].current;
       if(stats[i].rate < 0.0) stats[i].rate = 0.0;
       break;
     /* 
      * TEMP1 is a straight value.
      */
     case TEMP1:
       stats[i].rate = stats[i].current;
       if(stats[i].rate < 0.0) stats[i].rate = 0.0;
       break;
     /* 
      * TEMP2 is a straight value.
      */
     case TEMP2:
       stats[i].rate = stats[i].current;
       if(stats[i].rate < 0.0) stats[i].rate = 0.0;
       break;
     /* 
      * TEMP3 is a straight value.
      */
     case TEMP3:
       stats[i].rate = stats[i].current;
       if(stats[i].rate < 0.0) stats[i].rate = 0.0;
       break;
     /* 
      * FAN is a straight value.
      */
     case FAN:
       stats[i].rate = stats[i].current;
       if(stats[i].rate < 0.0) stats[i].rate = 0.0;
       break;
     /* 
      * FAN1 is a straight value.
      */
     case FAN1:
       stats[i].rate = stats[i].current;
       if(stats[i].rate < 0.0) stats[i].rate = 0.0;
       break;
     /* 
      * FAN2 is a straight value.
      */
     case FAN2:
       stats[i].rate = stats[i].current;
       if(stats[i].rate < 0.0) stats[i].rate = 0.0;
       break;
     /* 
      * FAN3 is a straight value.
      */
     case FAN3:
       stats[i].rate = stats[i].current;
       if(stats[i].rate < 0.0) stats[i].rate = 0.0;
       break;
     default:
       break;
   } /* End case switch */

 } /* End loop through enumerated types */

} /* End eval_proc_sensors() */
