%define name procstatd
%define version 1.4
%define release 2
%define buildroot /tmp/%{name}-%{version}.%{release}-root

%define builddir $RPM_BUILD_DIR/%{name}-%{version}.%{release}

Summary: A combined package of procstatd and watchman

Name: %{name}
Version: %{version}
Release: %{release}
Group: Beowulf
Copyright: GPL v2b
Source: %{name}-%{version}.%{release}.tgz
Buildroot: %{buildroot}

%description
procstatd is yet another proc-monitoring daemon designed to support
various sorts of beowulfery.  It is typically run under inetd and
tcp wrappers for security.  It is very lightweight -- it tries to be
as efficient as possible in parsing critical statistics from /proc
and packing them into a minimal human-readable packet.  The enclosed
perl-tk application "watchman" is just one way of monitoring the
output of procstatd.

%prep
%setup -q -n %{name}-%{version}.%{release}

%build
make clean
make

%install
make PREFIX=%{buildroot}/usr install

%files 
# The daemon
%attr(755,root,root) /usr/sbin/procstatd
# The man pages
%attr(644,root,root) /usr/share/man/man8/procstatd.8.gz
# The doc files
%attr(644,root,root) /usr/share/doc/procstatd/README
%attr(644,root,root) /usr/share/doc/procstatd/FAQ
%attr(755,root,root) /usr/share/doc/procstatd/debug_lm-sensors
%attr(644,root,root) /usr/share/doc/procstatd/CHANGES
%attr(644,root,root) /usr/share/doc/procstatd/COPYING


%clean
rm -rf $RPM_BUILD_ROOT
rm -rf %{builddir}

%post

# This is getting unpleasantly complex because it should be built to work
# both on old (inetd) systems and on new (xinetd) systems.  We therefore
# start with a conditional on which one we've got.

if [ -f /etc/inetd.conf ]
then
  # presumably we are an old/inetd system.
  if grep "^procstatd" /etc/inetd.conf > /dev/null
  then
    true
  else
    # There is no procstatd line installed.  Install one.
    cat << EOT
Backing up: 
/etc/inetd.conf -> /etc/inetd.conf.$$.rpmsave
/etc/services -> /etc/services.$$.rpmsave
/etc/hosts.allow -> /etc/hosts.allow.$$.rpmsave
Installing:
EOT
    cp /etc/inetd.conf /etc/inetd.conf.$$.rpmsave
    cp /etc/inetd.conf /tmp/inetd.conf.$$.tmp
    cat >> /tmp/inetd.conf.$$.tmp << EOT
# procstatd support
procstatd    stream  tcp     nowait  nobody    /usr/sbin/tcpd   /usr/sbin/procstatd -i 7885
EOT
    cp /etc/services /tmp/services.$$.tmp
    cp /etc/services /etc/services.$$.rpmsave
    cat >> /tmp/services.$$.tmp << EOT
procstatd       7885/tcp        # Private procstatd connection
EOT
    cp /etc/hosts.allow /tmp/hosts.allow.$$.tmp
    cp /etc/hosts.allow /etc/hosts.allow.$$.rpmsave
#
# NOTE:  For a local rebuild of the rpm, FIRST edit the allowed LAN's
# below (unless 192.168 is already correct and you want me to be
# able to monitor your beowulf from here:-)   rgb
#
   cat >> /tmp/hosts.allow.$$.tmp << EOT
procstatd:      192.168.,172.16.,10.
EOT

    mv /tmp/inetd.conf.$$.tmp /etc/inetd.conf
    mv /tmp/services.$$.tmp /etc/services
    mv /tmp/hosts.allow.$$.tmp /etc/hosts.allow

    killall -HUP inetd

  fi

  cat << EOT
Done:

  Congratulations!  procstatd is now installed, BUT:

    a) By default, procstatd allows tcp-wrapped access only from private
internal network addresses (10.0.0.0, 172.16.0.0, 192.168.0.0) which
should only exist "inside" gateway of some sort and which do not
generally route.  Edit /etc/hosts.allow to permit additional hosts or
networks to connect to procstatd.  Alternatively, change the entry to
ALL to allow promiscuous access and control access with e.g.  ipchains.
Restart inetd (and ipchains or whatever) when done with e.g.

  killall -HUP inetd

or

  /etc/rc.d/init.d/inet restart
  /etc/rc.d/init.d/ipchains restart

    b) If you want to install webwatch (the web version of watchman)
please retrieve the procstatd tgz or src.rpm and unpack its directory.  
It has to be hand configured and installed at this point, sorry.

    c) Be sure to read the README/FAQ/COPYING in /usr/doc/procstatd
and/or the man pages for procstatd.  I've put a fair amount of effort
into documentation, so please use it when possible to answer questions.

    d) As of this revision, watchman is unbundled.  If desired, install
watchman (which requires both procstatd and perl-Tk).

If you have problems that the documentation doesn't resolve, I'm
generally available at:

   Robert G. Brown <rgb@phy.duke.edu>

and there is a mailman-mediated procstatd mailing list accessible at:

   http://lists.phy.duke.edu/mailman/listinfo/procstatd

Good Luck.

EOT

elif [ -f /etc/xinetd.conf ]
then
  # This must be an xinetd system.  This should actually make it a
  # bit easier to install.

  # Save the old xinetd configuration if it exists so it can be replaced.
  if [ -f /etc/xinetd.d/procstatd ]
  then
     echo "Warning:  Saving old /etc/xinetd.d/procstatd in /etc/xinetd.d/procstatd.$$.rpmsave"
     /bin/mv /etc/xinetd.d/procstatd /etc/xinetd.d/procstatd.$$.rpmsave
  fi
  cat > /etc/xinetd.d/procstatd << EOF
# default: off
# description:  The procstatd daemon, to be managed by xinetd.
# Uncomment the "only_from" line below and edit it to permit access to
# private internal network hosts and/or specific networks.  Or leave
# it as is and control via ipchains.  The default to answer anybody who 
# asks...
service procstatd
{
	disable	= no
	socket_type		= stream
	protocol		= tcp
	wait			= no
	user			= nobody
	group			= nobody
	server			= /usr/sbin/procstatd
	server_args		= -i 7885
	port			= 7885
#	only_from		= 10.0.0.0,172.16.0,0,192.168.0.0
}
EOF

  killall -USR1 xinetd

  cat << EOF
Done:

  Congratulations!  procstatd is now installed, BUT:

    a) By default, procstatd allows tcp-wrapped access only from private
internal network addresses (10.0.0.0, 172.16.0.0, 192.168.0.0) which
should only exist "inside" gateway of some sort and which do not
generally route.  Edit /etc/xinetd.d/procstatd to permit additional
hosts or networks to connect to procstatd.  Alternatively, delete the
only_from line to allow promiscuous access and control access with e.g.
ipchains.

Do a soft or hard restart of xinetd when done by any
of:
  killall -USR1 xinetd           (soft)
  killall -USR2 xinetd           (hard -- forces immediate compliance 
                                  with access rules on running connections)
or

  /etc/rc.d/init.d/xinetd restart
  /etc/rc.d/init.d/ipchains restart

    b) If you want to install webwatch (the web version of watchman)
please retrieve the procstatd tgz or src.rpm and unpack its directory.  
It has to be hand configured and installed at this point, sorry.

    c) Be sure to read the README/FAQ/COPYING in /usr/doc/procstatd
and/or the man pages for procstatd.  I've put a fair amount of effort
into documentation, so please use it when possible to answer questions.

    d) As of this revision, watchman is unbundled.  If desired, install
watchman (which requires both procstatd and perl-Tk).

If you have problems that the documentation doesn't resolve, I'm
generally available at:

   Robert G. Brown <rgb@phy.duke.edu>

and there is a mailman-mediated procstatd mailing list accessible at:

   http://lists.phy.duke.edu/mailman/listinfo/procstatd

Good Luck.

EOF

else
  # Don't presume that there is xinetd just because there is no inetd.conf.
  cat << EOF
Help!  

I cannot figure out how to install procstatd on your system because you
don't seem to have either /etc/inetd.conf or /etc/xinetd.conf.  If you
want to run procstatd as a forking daemon (which is fine but which has
no intrinsic access control) you might want to add a script starting it
on whatever port you desire to e.g. /etc/rc.d/init.d/procstatd.

I'd recommend using tcpd to control access as well, although procstatd is
possibly harmless even if granted promiscuous access.
EOF
fi

exit

%postun

# This is getting unpleasantly complex because it should be built to work
# both on old (inetd) systems and on new (xinetd) systems.  We therefore
# start with a conditional on which one we've got.

if [ -f /etc/inetd.conf ]
then
  # presumably we are an old/inetd system.
  if grep "^procstatd" /etc/inetd.conf > /dev/null
  then
    # There is a procstatd line installed.  Remove it (and the
    # rest of the procstatd modifications).  Should be no need to
    # backup the files as I'm just going to sed them outta there.
    sed -e "/^# procstatd/d;/^procstatd/d" /etc/inetd.conf > /tmp/inetd.conf.$$.tmp
    sed -e "/^procstatd/d" /etc/services > /tmp/services.$$.tmp
    sed -e "/^procstatd/d" /etc/hosts.allow > /tmp/hosts.allow.$$.tmp
    mv /tmp/inetd.conf.$$.tmp /etc/inetd.conf
    mv /tmp/services.$$.tmp /etc/services
    mv /tmp/hosts.allow.$$.tmp /etc/hosts.allow
    killall -HUP inetd
  fi
elif [ -f /etc/xinetd.conf ]
then
  # This must be an xinetd system.  This should actually make it very
  # easy to uninstall

  # Save the old xinetd configuration if it exists so it can be replaced.
  if [ -f /etc/xinetd.d/procstatd ]
  then
     cat << EOF
Warning:  Saving old /etc/xinetd.d/procstatd in /etc/xinetd.d/procstatd.$$.rpmsave

Please remove this by hand as cruft if you are truly finished with procstatd
or archive it if you might ever need it again.
EOF
     /bin/mv /etc/xinetd.d/procstatd /etc/xinetd.d/procstatd.$$.rpmsave
  fi
else
  # Don't presume that there is xinetd just because there is no inetd.conf.
  cat << EOF
Help!  

I cannot figure out how to install procstatd on your system because you
don't seem to have either /etc/inetd.conf or /etc/xinetd.conf.  If you
want to run procstatd as a forking daemon (which is fine but which has
no intrinsic access control) you might want to add a script starting it
on whatever port you desire to e.g. /etc/rc.d/init.d/procstatd.

I'd recommend using tcpd to control access as well, although procstatd is
possibly harmless even if granted promiscuous access.
EOF
fi

%changelog
* Wed Dec 8 1999 Robert G. Brown <rgb@duke.edu>
- set up and built for 6.1
